import java.io.*;

public class BlockInputStream {
    int offset;
    DataInput input;

    public BlockInputStream(DataInput di) {
	input = di;
    }

    public BlockInputStream(InputStream is) {
	if (is instanceof DataInput) 
	    input = (DataInput) is;
	else
	    input = new DataInputStream(is);
    }

    public int readUnsignedByte() throws IOException {
	int b = input.readUnsignedByte();
	offset++;
	return b;
    }

    public int readUnsignedShort() throws IOException {
	return readUnsignedByte() + (readUnsignedByte() << 8);
    }

    public int readInt() throws IOException {
	return readUnsignedByte() + (readUnsignedByte() << 8)
	    + (readUnsignedByte() << 16) + (readUnsignedByte() << 24);
    }
    public long readUnsignedInt() throws IOException {
	return readUnsignedByte() + (readUnsignedByte() << 8)
	    + (readUnsignedByte() << 16) + ((long)readUnsignedByte() << 24);
    }
    public long readUnsignedLong() throws IOException {
	return readUnsignedByte() + (readUnsignedByte() << 8)
	    + (readUnsignedByte() << 16) + ((long)readUnsignedByte() << 24)
	    + ((long)readUnsignedByte() << 32) + ((long)readUnsignedByte() << 40)
	    + ((long)readUnsignedByte() << 48) + ((long)readUnsignedByte() << 56);
    }

    public long readVarInt() throws IOException {
	int first = readUnsignedByte();
	if (first < 0xfd) {
	    return first;
	} else if (first == 0xfd) {
	    return readUnsignedShort();
	} else if (first == 0xfe) {
	    return readUnsignedInt();
	} else if (first == 0xff) {
	    return readUnsignedLong();
	}
	throw new InternalError();
    }

    public void readFully(byte[] arr) throws IOException {
	input.readFully(arr);
	this.offset += arr.length;
    }


    public void skipFully(int toskip) throws IOException {
	this.offset += toskip;
	while (toskip > 0) {
	    toskip -= input.skipBytes(toskip);
	}
    }

    public int getOffset() {
	return offset;
    }
}