import java.io.*;
import java.math.*;
import java.security.*;
import java.util.*;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

public class KeyUtil {
    SecP256k secp256k = new SecP256k();
    MessageDigest sha256, rip160;
    
    public KeyUtil() {
	try {
	    Security.addProvider(new BouncyCastleProvider());
	    sha256 = MessageDigest.getInstance("SHA-256");
	    rip160 = MessageDigest.getInstance("RIPEMD160");
	} catch (NoSuchAlgorithmException ex) {
	    throw new RuntimeException(ex);
	}
    }

    public byte[] sha256(byte[] data) {
	sha256.reset();
	sha256.update(data);
	return sha256.digest();
    }
    public byte[] rip160(byte[] data) {
	rip160.reset();
	rip160.update(data);
	return rip160.digest();
    }

    public byte[] pk2hash(byte[] pubkey) {
	byte[] digest = rip160(sha256(pubkey));
	byte[] hash = new byte[digest.length + 1];
	System.arraycopy(digest, 0, hash, 1, digest.length);
	return hash;
    }

    public String base58encode(byte[] raw) {
	byte[] arr = new byte[raw.length + 4];
	byte[] digest = sha256(sha256(raw));
	System.arraycopy(raw, 0, arr, 0, raw.length);
	System.arraycopy(digest, 0, arr, raw.length, 4);

	String table = 
	    "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz";
	BigInteger i = new BigInteger(1, arr);
	StringBuilder sb = new StringBuilder();
	BigInteger base = BigInteger.valueOf(58);
	while (i.signum() > 0) {
	    int digit = (int)i.mod(base).longValue();
	    sb.append(table.charAt(digit));
	    i = i.divide(base);
	}
	for (int j = 0; arr[j] == 0 && j < arr.length; j++)
	    sb.append('1');
	sb.reverse();
	return sb.toString();
    }

    public byte[] base58decode(String key) {
	String table = 
	    "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz";
	BigInteger bin = BigInteger.ZERO;
	BigInteger base = BigInteger.valueOf(58);
	int i = 0;
	while (i < key.length() && key.charAt(i) == '1')
	    i++;
	int zeroes = i;
	while (i < key.length()) {
	    int v = table.indexOf(key.charAt(i));
	    if (v < 0)
		return null;
	    bin = bin.multiply(base).add(BigInteger.valueOf(v));
	    i++;
	}
	byte[] binary = bin.toByteArray();
	int len = binary.length - 4;
	int offset = 0;
	if (binary[0] == 0) {
	    offset ++;
	    len--;
	}
	byte[] raw = new byte[len + zeroes];
	System.arraycopy(binary, offset, raw, zeroes, len);
	sha256.reset();
	sha256.update(raw, 0, raw.length);
	byte[] digest = sha256.digest();
	sha256.reset();
	sha256.update(digest);
	digest = sha256.digest();
	for (i = 0; i < 4; i++) {
	    if (binary[offset + len + i] != digest[i]) {
		System.err.println("Checksum failed for "+key);
		return null;
	    }
	}
	return raw;
    }
}
