import java.util.ArrayList;

public class Script {
    static class Operation {
	String name;
	public Operation(String n) {
	    name = n;
	}

	public byte[] getPushData() {
	    throw new UnsupportedOperationException();
	}

	public String toString() {
	    return name;
	}
    }

    public static final Operation OP_DUP = new Operation("OP_DUP");
    public static final Operation OP_SWAP = new Operation("OP_SWAP");
    public static final Operation OP_NOT = new Operation("OP_NOT");
    public static final Operation OP_BOOLOR = new Operation("OP_BOOLOR");
    public static final Operation OP_RETURN = new Operation("OP_RETURN");
    public static final Operation OP_HASH160 = new Operation("OP_HASH160");
    public static final Operation OP_HASH256 = new Operation("OP_HASH256");
    public static final Operation OP_EQUAL = new Operation("OP_EQUAL");
    public static final Operation OP_EQUALVERIFY = new Operation("OP_EQUALVERIFY");
    public static final Operation OP_CHECKSIG = new Operation("OP_CHECKSIG");
    public static final Operation OP_CHECKSIGVERIFY = new Operation("OP_CHECKSIGVERIFY");
    public static final Operation OP_CHECKMULTISIG = new Operation("OP_CHECKMULTISIG");
    public static final Operation OP_CHECKMULTISIGVERIFY = new Operation("OP_CHECKMULTISIGVERIFY");


    static class PushOperation extends Operation {
	byte[] data;

	public PushOperation(byte[] d) {
	    super("OP_PUSH");
	    data = d;
	}
	public String toString() {
	    return Util.hexBytes(data);
	}

	public byte[] getPushData() {
	    return data;
	}
    }

    Operation[] ops;

    public Script(byte[] script) {
	ArrayList<Operation>  opcodes = new ArrayList<Operation>();
	for (int i = 0; i< script.length; i++) {
	    int opc = script[i] & 0xff;
	    switch (opc) {
	    case 76: {
		int len;
		if (script.length < i+2) {
		    len = script.length - i - 2;
		} else {
		    len = script[i+1]&0xff;
		    if (len > script.length - i - 2)
			len = script.length - i - 2;
		    byte data[] = new byte[len];
		    System.arraycopy(script, i+2, data, 0, len);
		    opcodes.add(new PushOperation(data));
		}
		i += len + 1;
		break;
	    }
	    case 77: {
		int len;
		if (script.length < i+3) {
		    len = script.length - i - 3;
		} else {
		    len = (script[i+1]&0xff) + ((script[i+2]&0xff) << 8);
		    if (len > script.length - i - 3)
			len = script.length - i - 3;
		    byte data[] = new byte[len];
		    System.arraycopy(script, i+3, data, 0, len);
		    opcodes.add(new PushOperation(data));
		}
		i += len + 2;
		break;
	    }
	    case 78: {
		int len;
		if (script.length < i+5) {
		    len = script.length - i - 5;
		} else {
		    len = (script[i+1]&0xff) + ((script[i+2]&0xff) << 8)
			+ ((script[i+3]&0xff) << 16) + ((script[i+4]&0xff) << 24);
		    if (len < 0)
			len = 0;
		    if (len > script.length - i - 5)
			len = script.length - i - 5;
		    byte data[] = new byte[len];
		    System.arraycopy(script, i+5, data, 0, len);
		    opcodes.add(new PushOperation(data));
		}
		i += len + 4;
		break;
	    }
	    case 106:
		opcodes.add(OP_RETURN);
		break;
	    case 118:
		opcodes.add(OP_DUP);
		break;
	    case 124:
		opcodes.add(OP_SWAP);
		break;
	    case 145:
		opcodes.add(OP_NOT);
		break;
	    case 155:
		opcodes.add(OP_BOOLOR);
		break;
	    case 169:
		opcodes.add(OP_HASH160);
		break;
	    case 170:
		opcodes.add(OP_HASH256);
		break;
	    case 135:
		opcodes.add(OP_EQUAL);
		break;
	    case 136:
		opcodes.add(OP_EQUALVERIFY);
		break;
	    case 172:
		opcodes.add(OP_CHECKSIG);
		break;
	    case 173:
		opcodes.add(OP_CHECKSIGVERIFY);
		break;
	    case 174:
		opcodes.add(OP_CHECKMULTISIG);
		break;
	    case 175:
		opcodes.add(OP_CHECKMULTISIGVERIFY);
		break;
	    default:
		if (opc <= 75) {
		    int len = opc;
		    if (len > script.length - i - 1)
			len = script.length - i - 1;
		    byte data[] = new byte[len];
		    System.arraycopy(script, i+1, data, 0, len);
		    opcodes.add(new PushOperation(data));
		    i += len;
		} else if (opc == 79 || (opc >= 81 && opc <= 96)) {
		    byte data[] = new byte[] { (byte) (opc - 80) };
		    opcodes.add(new PushOperation(data));
		} else {
		    System.err.println("FIXME: Unknown opcode: " + opc);
		    opcodes.add(new Operation("OP_UNKNOWN"+opc));
		}
	    }
	}
	ops = opcodes.toArray(new Operation[opcodes.size()]);
    }

    public String toString () {
	StringBuilder sb = new StringBuilder("Script[");
	String comma = "";
	for (Operation op: ops) {
	    sb.append(comma).append(op);
	    comma = " ";
	}
	sb.append("]");
	return sb.toString();
    }

    public Operation getLast() {
	return ops[ops.length-1];
    }

    public void removeLast() {
	Operation[] newOps = new Operation[ops.length-1];
	System.arraycopy(ops, 0, newOps, 0, ops.length-1);
	ops = newOps;
    }

    public boolean isPayToScriptHash() {
	return ops.length == 3
	    && ops[0] == OP_HASH160
	    && ops[1] instanceof PushOperation
	    && ops[2] == OP_EQUAL;
    }

    public boolean isPayToHash() {
	return ops.length == 5
	    && ops[0] == OP_DUP
	    && ops[1] == OP_HASH160
	    && ops[2] instanceof PushOperation
	    && ops[3] == OP_EQUALVERIFY
	    && ops[4] == OP_CHECKSIG;
    }

    public boolean isCheckSig() {
	return ops.length == 2
	    && ops[0] instanceof PushOperation
	    && ops[1] == OP_CHECKSIG;
    }

    public boolean isOpReturn() {
	return ops.length >= 1
	    && ops[0] == OP_RETURN;
    }
    
    public boolean isCheckMultiSig() {
	if (ops.length < 3 || ops[ops.length-1] != OP_CHECKMULTISIG)
	    return false;
	for (int i = 0; i < ops.length-1; i++) {
	    if (!(ops[i] instanceof PushOperation))
		return false;
	}
	byte[] numsigs = ops[ops.length-2].getPushData();
	if (numsigs.length != 1 || ops.length != numsigs[0] + 3)
	    return false;
	byte[] needed = ops[0].getPushData();
	if (needed.length != 1 || (needed[0] & 0xff) > (numsigs[0] & 0xff))
	    return false;
	return true;
    }
}
