import java.net.URL;
import java.net.HttpURLConnection;
import java.io.*;
import java.math.BigInteger;
import java.util.Properties;
import javax.xml.bind.DatatypeConverter;

public class TransactionReader {
    String url = "http://127.0.0.1:8332/";
    String authentication;

    public TransactionReader(String config) throws IOException {
	Properties props = new Properties();
	props.load(new FileReader(config));
	String userpw = props.getProperty("rpcuser") + ":" +
	    props.getProperty("rpcpassword");
	authentication = "Basic " +
	    DatatypeConverter.printBase64Binary(userpw.getBytes("UTF-8"));
    }

    public Transaction getTransaction(byte[] txid) throws IOException {
	String txhex = Util.hexBytes(Util.reverse(txid));
	String request = "{\"method\":\"getrawtransaction\","+
	    "\"params\":[\""+txhex+"\"],\"id\":1}\n";
	byte[] reqBytes = request.getBytes();
	HttpURLConnection conn = (HttpURLConnection)
	    new URL(url).openConnection();
	conn.setDoOutput(true);
	conn.setRequestMethod("POST");
	
	conn.setRequestProperty("Authorization", authentication);
	conn.setRequestProperty("Content-Length", Integer.toString(reqBytes.length));
	OutputStream os = conn.getOutputStream();
	os.write(reqBytes);
	os.close();
	InputStream is = conn.getInputStream();
	BufferedReader reader = new BufferedReader(new InputStreamReader(is));
	String line = reader.readLine();
	reader.close();
	if (!line.substring(0, 11).equals("{\"result\":\""))
	    throw new IOException("Wrong format: "+line);
	String rawhex = line.substring(11, line.indexOf('"', 11));
	byte[] rawtx = Util.hexToBytes(rawhex);
	Transaction tx = new Transaction();
	tx.read(new BlockInputStream(new ByteArrayInputStream(rawtx)));
	return tx;
    }

    public static void main(String[] args) throws IOException {
	if (args.length == 0) {
	    System.err.println("USAGE: java TransactionReader <bitcoin.config>");
	    return;
	}
	TransactionReader txreader = new TransactionReader(args[0]);
	String txid =
	    "9e292942338bd52b61a793b01537f52e8d1ac644a2556e2d1f57bcaee529cf73";
	Transaction tx = txreader.getTransaction(Util.reverse(Util.hexToBytes(txid)));
	System.err.println(tx);
    }
}
