import java.util.ArrayList;

public class Util {
    public static String hexBytes(byte[] bytes, int offset, int len) {
	StringBuilder sb = new StringBuilder();
	for (int i = offset; i < offset+len; i++) {
	    sb.append(Integer.toString(0x100+(bytes[i]&0xff), 16).substring(1));
	}
	return sb.toString();
    }

    public static String hexBytes(byte[] bytes) {
	return hexBytes(bytes, 0, bytes.length);
    }

    public static byte[] hexToBytes(String hex) {
	if (hex.length() % 2 == 1)
	    hex = "0" + hex;
	byte[] raw = new byte[hex.length()/2];
	for (int i = 0; i< raw.length; i++) {
	    raw[i] = (byte) Integer.parseInt(hex.substring(2*i, 2*i+2), 16);
	}
	return raw;
    }

    public static byte[] reverse(byte[] bytes) {
	byte[] result = new byte[bytes.length];
	for (int i = 0; i < bytes.length; i++)
	    result[i] = bytes[bytes.length-1-i];
	return result;
    }

    public static byte[][] decodeDERSignature(byte[] sig) {
	if (sig.length < 4
	    || sig[0] != 0x30 || (sig[1] & 0xff) != sig.length - 3
	    || sig[2] != 2)
	    return null;
	int rlen = sig[3]&0xff;
	if (sig.length < 6+rlen
	    || sig[rlen+4] != 2
	    || sig[rlen+5] != sig.length  - rlen - 7)
	    return null;
	int slen = sig[rlen+5] & 0xff;
	byte[] r = new byte[32];
	if (rlen <= 32)
	    System.arraycopy(sig, 4, r, 32-rlen, rlen);
	else if (rlen > 33 || sig[4] != 0)
	    return null;
	else
	    System.arraycopy(sig, 5, r, 0, 32);
	byte[] s = new byte[32];
	if (slen <= 32)
	    System.arraycopy(sig, rlen+6, s, 32-slen, slen);
	else if (slen > 33 || sig[rlen+6] != 0)
	    return null;
	else
	    System.arraycopy(sig, rlen+7, s, 0, 32);
	return new byte[][] {r,s};
    }

    public static byte[][] decodeScriptInput(byte[] input) {
	int pos = 0;
	ArrayList<byte[]> inputs= new ArrayList<byte[]>();
	while (pos < input.length) {
	    byte[] data;
	    int offset, len;
	    if ((input[pos] & 0xff) <= 75) {
		offset = pos+1;
		len = input[pos];
	    } else if (input[pos] == 76) {
		offset = pos+2;
		len = (input[pos+1]&0xff);
	    } else if (input[pos] == 77) {
		offset = pos+3;
		len = (input[pos+1]&0xff) + ((input[pos+2]&0xff) << 8);
	    } else if (input[pos] >= 81 && input[pos] <= 96) {
		data = new byte[] { (byte) (input[pos] - 80) };
		inputs.add(data);
		pos++;
		continue;
	    } else {
		return null;
	    }
	    data = new byte[len];
	    System.arraycopy(input, offset, data, 0, len);
	    pos = offset + len;
	    inputs.add(data);
	}
	return inputs.toArray(new byte[inputs.size()][]);
    }
}
